@@ -1,67 +1,40 @@
-import httpx
+from httpx import AsyncClient
 import ujson
 import asyncio
-from typing import Any, Dict, List, Optional, Tuple, Union
-from functools import wraps
-from idac_sdk.models.vpn_config import VPNConfig, VPNType
+from typing import Dict, List, Optional, Tuple, Union
 
-from idac_sdk.types import IDACRequestStatus, IDACRequestType
+from idac_sdk.base import (
+    DEFAULT_WAIT_INTERVAL,
+    DEFAULT_WAIT_TIMEOUT,
+    REQUEST_ERROR_STATES,
+    REQUEST_GOOD_STATES,
+)
+from idac_sdk.base.request import BaseIDACRequest, check_controller
+from idac_sdk.types import IDACRequestStatus, IDACRequestType, IDACAuthType
 from idac_sdk.log import logger
-from . import DEFAULT_USER_AGENT, DEFAULT_WAIT_INTERVAL, DEFAULT_WAIT_TIMEOUT
-
-from idac_sdk.asynced.controller import IDACAuthType, IDACController
+from idac_sdk.asynced.controller import IDACController as AsyncController
 from idac_sdk.errors import (
     IDACRequestStateError,
     IncorrectMinutesValue,
-    IncorrectWantedStateError,
     NoAuth,
-    NoControllerError,
     NoIdError,
-    UnknownVPNType,
 )
+from idac_sdk.models.vpn_config import VPNConfig
 from idac_sdk.models.request_state import RequestState
 from idac_sdk.session_data import SessionData
 
 
-def check_controller(method):
-    @wraps(method)
-    def wrapper(self, *method_args, **method_kwargs):
-        if not isinstance(self.controller, IDACController):
-            raise NoControllerError("iDAC controller not provided")
-        return method(self, *method_args, **method_kwargs)
-
-    return wrapper
-
-
-REQUEST_ERROR_STATES = [
-    IDACRequestStatus.cancelled,
-    IDACRequestStatus.error,
-    IDACRequestStatus.onboardError,
-]
-REQUEST_GOOD_STATES = [
-    IDACRequestStatus.active,
-    IDACRequestStatus.executed,
-    IDACRequestStatus.complete,
-]
-
-
-class IDACRequest:
-    session_data: Optional[SessionData]
-    controller: IDACController
-    uuid: Optional[str]
-    user_agent: str
-    vpn: VPNConfig
-
+class IDACRequest(BaseIDACRequest[AsyncController]):
     def __init__(
         self,
         uuid: Optional[str] = None,
         session_data: Optional[SessionData] = None,
-        controller: Optional[IDACController] = None,
+        controller: Optional[AsyncController] = None,
         user_agent: Optional[str] = None,
         vpn: Optional[VPNConfig] = None,
     ) -> None:
         """
-        IDACRequest Object
+        Async IDACRequest Object
 
         IDACRequest describes iDAC request object.
         Handles all operations with requests: create, cleanup, restart.
@@ -76,31 +49,16 @@ class IDACRequest:
             user_agent (Optional[str], optional): User-Agent string. Defaults to None.
             vpn (Optional[VPNConfig], optional): VPN options
         """
-        if not uuid:
-            # no UUID -> new request, need to set session_data and controller
-            self.session_data = SessionData() if not session_data else session_data
-            self.uuid = None
-        else:
-            # UUID provided -> existing request
-            # self.session_data = None
-            self.session_data = SessionData()
-            self.uuid = uuid
-
-        self.user_agent = user_agent if user_agent else DEFAULT_USER_AGENT
-
-        if not controller and controller is not False:
-            if isinstance(self.session_data, SessionData) and self.session_data.has("creds"):
-                self.controller = IDACController(
-                    auth_type=IDACAuthType.DCLOUD_SESSION,
-                    auth=self.session_data.get("creds"),
-                )
-            else:
-                self.controller = IDACController()
-        else:
-            self.controller = controller
-
-        self.vpn = vpn or self.controller.vpn
+        super().__init__(
+            uuid=uuid,
+            session_data=session_data,
+            controller=controller,
+            user_agent=user_agent,
+            vpn=vpn,
+            wanted_controller_type=AsyncController,
+        )
 
+    @check_controller
     async def check_controller_auth(self) -> None:
         """Tells controller to request auth token if auth type is DCLOUD_SESSION"""
         logger.debug(f"Checking auth. Type is {self.controller.auth.type}")
@@ -116,51 +74,6 @@ class IDACRequest:
             token = await self.controller.get_auth_token()
             logger.debug(f"Worker token is: {token}")
 
-    def add_vpn(self, where: dict[str, Any]) -> dict[str, Any]:
-        """Adds VPN parameters to request
-
-        Args:
-            where (dict[str, Any]): Where VPN params should be added
-
-        Raises:
-            UnknownVPNType: Raised if unknown VPN type configured
-
-        Returns:
-            dict[str, Any]: updated dictionary with VPN parameters
-        """
-        logger.debug("Adding VPN parameters: %s", self.vpn)
-        if ("idacVpn" in where) or (self.vpn.type == VPNType.none):
-            return where
-
-        if self.vpn.type == VPNType.vpod:
-            # adding params for vPod VPN. `datacenter`, `vpod` and `anycpwd` should be loaded from session.xml
-            where.update({"idacVpn": "dcloud-vpod"})
-        elif self.vpn.type == VPNType.request or self.vpn.type == VPNType.explicit:
-            # adding params for `request` - explicit VPN settings
-            where.update(
-                {
-                    "idacVpn": "request",
-                    "idacVpnHost": self.vpn.params.host,
-                    "idacVpnUsername": self.vpn.params.username,
-                    "idacVpnPassword": self.vpn.params.password,
-                }
-            )
-        elif self.vpn.type == VPNType.secure_repo:
-            # adding params for `secure_repo` - VPN settings will be loaded from secure repo
-            where.update(
-                {
-                    "idacVpn": "secureRepo",
-                    "idacVpnSecureRepoSpace": self.vpn.params.secure_repo_space,
-                    "idacVpnSecureRepoFile": self.vpn.params.secure_repo_file,
-                    "idacVpnSecureRepoBlock": self.vpn.params.secure_repo_block,
-                    "idacVpnSecureRepoKey": self.vpn.params.secure_repo_key,
-                }
-            )
-        else:
-            raise UnknownVPNType("Unknown VPN type provided")
-
-        return where
-
     @check_controller
     async def create(
         self, request_type: IDACRequestType = IDACRequestType.SIMPLE
@@ -202,7 +115,7 @@ class IDACRequest:
 
         redirect = None
         st = None
-        async with httpx.AsyncClient() as client:
+        async with AsyncClient() as client:
             client.headers.update({"User-Agent": self.user_agent})
             r = await client.request(method, api, headers=headers, **kwargs)
             logger.debug(f"Got response: {r.status_code} {r.reason_phrase}")
@@ -241,7 +154,7 @@ class IDACRequest:
         headers: Dict[str, str] = {"Accept": "application/json"}
         self.controller.with_auth(headers)
 
-        async with httpx.AsyncClient() as client:
+        async with AsyncClient() as client:
             client.headers.update({"User-Agent": self.user_agent})
             r = await client.request(method, api, headers=headers)
             if r.status_code < 200 or r.status_code > 399:
@@ -267,7 +180,7 @@ class IDACRequest:
         headers: Dict[str, str] = {"Accept": "application/json"}
         self.controller.with_auth(headers)
 
-        async with httpx.AsyncClient() as client:
+        async with AsyncClient() as client:
             client.headers.update({"User-Agent": self.user_agent})
             r = await client.request(method, api, headers=headers)
             logger.debug(f"Got response: {r.status_code} {r.reason_phrase}")
@@ -292,7 +205,7 @@ class IDACRequest:
         headers: Dict[str, str] = {"Accept": "application/json"}
         self.controller.with_auth(headers)
 
-        async with httpx.AsyncClient() as client:
+        async with AsyncClient() as client:
             client.headers.update({"User-Agent": self.user_agent})
             r = await client.request(method, api, headers=headers)
             logger.debug(f"Got response: {r.status_code} {r.reason_phrase}")
@@ -317,7 +230,7 @@ class IDACRequest:
         headers: Dict[str, str] = {"Accept": "application/json"}
         self.controller.with_auth(headers)
 
-        async with httpx.AsyncClient() as client:
+        async with AsyncClient() as client:
             client.headers.update({"User-Agent": self.user_agent})
             r = await client.request(method, api, headers=headers)
             logger.debug(f"Got response: {r.status_code} {r.reason_phrase}")
@@ -347,7 +260,7 @@ class IDACRequest:
         headers: Dict[str, str] = {"Accept": "application/json"}
         self.controller.with_auth(headers)
 
-        async with httpx.AsyncClient() as client:
+        async with AsyncClient() as client:
             client.headers.update({"User-Agent": self.user_agent})
             r = await client.request(method, api, headers=headers)
             logger.debug(f"Got response: {r.status_code} {r.reason_phrase}")
@@ -358,11 +271,13 @@ class IDACRequest:
     @check_controller
     async def wait_for_status(
         self,
-        wanted_state: List[Union[IDACRequestStatus, str]] = REQUEST_GOOD_STATES,  # type: ignore
+        wanted_state: Union[
+            List[IDACRequestStatus], List[str], List[Union[IDACRequestStatus, str]]
+        ] = REQUEST_GOOD_STATES,
         stop_on_error: bool = True,
         timeout: Optional[int] = DEFAULT_WAIT_TIMEOUT,
         interval: int = DEFAULT_WAIT_INTERVAL,
-    ) -> None:
+    ) -> Union[RequestState, None]:
         """Waits for request to land in one of wanted states by periodically checking it's status
 
         Args:
@@ -382,22 +297,16 @@ class IDACRequest:
         if not self.uuid:
             raise NoIdError("No ID/UUID provided for Wait For Status")
 
-        for idx, val in enumerate(wanted_state):
-            if isinstance(val, str):
-                wanted_state[idx] = IDACRequestStatus(val)
-            elif not isinstance(val, IDACRequestStatus):
-                raise IncorrectWantedStateError(f"Incorrect wanted state: {val}")
+        wanted_state = self.sanitize_states(wanted_state)
 
-        async def try_infinite():
+        async def try_infinite() -> RequestState:
             while True:
                 st = await self.get_state()
                 status = IDACRequestStatus(st.status)
                 if status in wanted_state:
-                    return
+                    return st
                 if stop_on_error and status in REQUEST_ERROR_STATES:
                     raise IDACRequestStateError(f"Request landed in error state: {status.value}")
                 await asyncio.sleep(interval)
 
-        await asyncio.wait_for(try_infinite(), timeout=timeout)
-
-        return
+        return await asyncio.wait_for(try_infinite(), timeout=timeout)
