@@ -1,5 +1,5 @@
 from __future__ import annotations
-from typing import Any, List, Optional
+from typing import Any, List, Literal, Optional, Union, cast
 from pydantic import BaseModel, Extra
 
 from idac_sdk.models.new_request import NewRequest
@@ -121,3 +121,15 @@ class RequestState(BaseModel, extra=Extra.allow):
     statusApi: Optional[str]
     worker: Optional[Worker]
     history: Optional[List[HistoryItem]]
+
+    def get_task_outputs(
+        self, section: Literal["Start", "Stop"], task_name: str
+    ) -> Union[dict[str, Any], None]:
+        if not hasattr(self.tasks, section):
+            raise KeyError(f"No section {section} in tasks")
+
+        for tsk in cast(list, getattr(self.tasks, section)):
+            if tsk["name"] == task_name:
+                return tsk["output"]
+
+        return None
